import debug

class Binding:
	def check_args(self,**args):
		" check arguments "
		for argname in self.argnames:
			if argname not in args.keys():
				debug.error(4,"BIND","'%s' is missing key %s"%(self.name,argname))
				return False
		if len(args) > len(self.argnames):
			debug.error(4,"BIND","to many arguments. '%s' requires %s"%(
				self.name, unicode(self.argnames) ) )
			return False
		return True

class Request(Binding):
	def __init__( self, name, *argnames ):
		self.name = name
		self.argnames = argnames
		self.bound = False
		self.__call__ = self.request
	def listen( self, function ):
		self.bound = function
	def remove( self, function ):
		if self.bound==function:
			self.bound = False
	def request( self, **args ):
		debug.info(8,"REQUEST", "'%s' is triggerd"%self.name)
		if self.bound \
		and self.check_args( **args ):
			return self.bound( **args )
		else:
			return False

class Requests(Binding):
	def __init__( self, name, *argnames ):
		self.name = name
		self.argnames = argnames
		self.bound = {}
		self.__call__ = self.request
	def listen( self, function, *extraargs ):
		self.bound[function] = extraargs
	def remove( self, function ):
		if function in self.bound.keys():
			del self.bound[function]
	def request(self,**args):
		debug.info(8,"MULTIREQUEST", "'%s' is triggerd"%self.name)
		if self.bound \
		and self.check_args(**args):
			# run request functions
			all_results = []
			targets = self.bound.copy()
			for (func,extra) in targets.iteritems():
				result = func( *extra, **args )
				if result:
					if type(result)==list:
						all_results.extend( result )
					else:
						all_results.append( result )
			return all_results
		else:
			return False

class Event(Binding):
	def __init__( self, name, *argnames ):
		self.name = name
		self.argnames = argnames
		self.bound = {}
		self.__call__ = self.trigger
	def listen( self, function, *extraargs ):
		self.bound[function] = extraargs
	def remove( self, function ):
		if function in self.bound.keys():
			del self.bound[function]
	def trigger(self,**args):
		debug.info(8,"EVENT", "'%s' is triggerd"%self.name)
		if self.bound \
		and self.check_args(**args):
			# run target functions
			targets = self.bound.copy()
			for (func,extra) in targets.iteritems():
				func( *extra, **args )

if __name__=="__main__":

	debug.set_info_level(9)
	debug.set_error_level(9)

	def func1(a,b):
		print "1:",a,b
		return a+b

	def func2(extra, a=0,b=0):
		print "2:",a,b,extra
		return a-b

	def func3(lista=False):
		lista.append("one_more")
		lista[0]=lista[0]+"_"
		return lista

	r=Requests("hej","a","b")
	r.listen(func1)
	r.listen(func2,"Gottgottigottgott")
	print r(a=1,b=20)

	r=Request("listchange","lista")
	r.listen(func3)
	a=['first']
	print r(lista=a)
	print a

	e=Event("tjo","a","b")
	e.listen(func1)
	e.listen(func2)
	e.remove(func2)
	e(a=1,b=20)
