## -*- encoding: utf8

"""
ToDo:

* fix dbus-calls so that an error is given
	if/when nm crashes (or aint started)
	( add try-cases. if not succeded, send update_status( Error )
	? when should it try to connect again? could it listen to
		NMsignals through dbus?

* Send client with calls. Use standard calls only changed if realy needed.
	on_something( client, device=None, network=None )

* work with new disk.Stored and/or
	remove dependencies on .__conf
	use "on_status_changed"

"""

# External libraries
import dbus

# Own libraries
from constants import *		# constants, Device and Wireless
import debug					# debugging and errors

# Client
NM_DBUS_SERVICE		= 'org.freedesktop.NetworkManager'
NM_DBUS_PATH		= '/org/freedesktop/NetworkManager'
NM_DBUS_IFACE 		= 'org.freedesktop.NetworkManager'


class Client:
	def __init__( self, main ):
		self.main = main

		self.main.activate_network.listen( self.activate )

		self.main.active_device.listen( self.active_device )
		self.main.active_network.listen( self.active_network )
		self.main.current_status.listen( self.current_status )

		self.main.available_devices.listen( self.list_devices )
		self.main.available_networks.listen( self.list_networks )

		self.connect_to_nm_dbus()

	def connect_to_nm_dbus( self ):
		self.__bus = dbus.SystemBus()
		iface = self.__get_nm_dbus_interface()

		if iface:
			"""
			Fix a better test. Should also automaticly
			connect if nm appeares on the dbus.
			"""
			iface.connect_to_signal(
				'DevicesChanged',self.onDevicesChanged)

			iface.connect_to_signal(
				'DeviceNoLongerActive',self.onDeviceNoLongerActive)
			iface.connect_to_signal(
				'DeviceActivating',self.onDeviceActivating)
			iface.connect_to_signal(
				'DeviceNowActive',self.onDeviceNowActive)

			iface.connect_to_signal(
				'DeviceStrengthChanged',self.onDeviceStrengthChanged)

			iface.connect_to_signal(
				'WirelessNetworkAppeared',self.onWirelessNetworkAppeared)
			iface.connect_to_signal(
			'WirelessNetworkDisappeared',self.onWirelessNetworkDissappeared)
		else:
			self.main.nm_error.trigger(
				message="Couldn't connect to NetworkManager")

	def __get_nm_dbus_object( self ):
		try:
			return self.__bus.get_object( NM_DBUS_SERVICE, NM_DBUS_PATH )
		except:
			debug.error( 2, "DBUS",
				"Couldnt get object, NetworkManager isnt running" )
			self.main.nm_error.trigger(
				message="Couldn't get NetworkManager object")
			return False

	def __get_nm_dbus_interface( self ):
		object = self.__get_nm_dbus_object()
		if object:
			try:
				return dbus.Interface( object, NM_DBUS_IFACE )
			except:
				debug.error( 2, "DBUS",
					"Couldnt get interface, NetworkManager isnt running" )
				self.main.nm_error.trigger(
					message="Couldn't get NetworkManager interface")
				return False

	# Signals
##############################################################

	def onDeviceNoLongerActive(self,dev_path):
		self.main.status_changed( status=PASSIVE, network=False )

	def onDeviceActivating(self,dev_path):
		self.main.status_changed( status=WAITING, network=False )

	def onDeviceNowActive(self,dev_path,essid=False):
		device=Device(dev_path)
		network = self.main.active_network.request( device=device )
		if not network:
			network=device
		self.main.status_changed( status=ACTIVE, network=network )

	def onDeviceStrengthChanged(self,dev_path,strength):
		self.main.strength_changed( strength=strength )

	def onWirelessNetworkAppeared(self,dev_path,net_path):
		network=Network( dev_path, net_path )
		self.network_complement( network )
		self.main.networks_altered( appeared=True, network=network )

	def onWirelessNetworkDissappeared(self,dev_path,net_path):
		self.main.networks_altered( appeared=False, network=False )
		#client=self, appeared=False, network=Network(dev_path,net_path) )

	def onDevicesChanged(self,*info):
		debug.info( 2, 'CLIENT', "Devices changed: "+unicode(info) )

	# Methods
###########################################################

	def list_devices( self ):
		" Return a list of all devices "
		res = []
		for dev_path in self.__get_nm_dbus_interface().getDevices():
			res.append( Device( dev_path ) )
		return res

	def list_networks( self, device ):
		" Return a list of all networks on a device "
		self.device_complement( device )
		res = []
		for net_path in device[NETWORK_PATHS]:
			res.append(
				self.network_complement(
					Network( device[DEVICE_PATH], net_path ) ) )
		return res


	def network_properties( self, net_path ):
		net_obj = self.__bus.get_object( NM_DBUS_SERVICE, net_path )
		return net_obj.getProperties()

	def network_complement( self, network ):
		return network.complement(
				self.device_properties( network[DEVICE_PATH] ),
				self.network_properties( network[NETWORK_PATH] ) )

	def device_properties( self, dev_path ):
		dev_obj = self.__bus.get_object( NM_DBUS_SERVICE, dev_path )
		return dev_obj.getProperties()

	def device_complement( self, device ):
		return device.complement(
			self.device_properties( device[DEVICE_PATH] ) )

	def active_device( self ):
		for device in self.list_devices():
			self.device_complement( device )
			if device[ACTIVE]:
				return device
		return None

	def current_status( self ):
		"Ask NM directly"
		if self.active_device():
			return ACTIVE
		else:
			return PASSIVE

	def active_network( self, device ):
		" Returns the active network on selected device or None "
		if not device:
			return None
		self.device_complement(device)
		if device[ACTIVE] and device[ACTIVE_NETWORK_PATH]:
			network = Network(
				device[DEVICE_PATH], device[ACTIVE_NETWORK_PATH] )
			self.network_complement( network )
			return network
		return None

	def activate( self, network=False ):
		" sets active network, usertriggerd, also set network trusted "

		if network[TYPE]==WIRELESS:
			try:
				self.__get_nm_dbus_object().setActiveDevice(
					dbus.ObjectPath( network[NETWORK_PATH] ),
					dbus.String( network[ESSID] ) )
			except dbus.DBusException,e:
				debug.error( 3, "CLIENT",
					"%s: %s"%(NAME_DEVICE_TYPE[WIRELESS],e) )
				return False
			network[TRUSTED]=True
			self.main.network_activated( network=network )
			debug.info( 2, "CLIENT",
				"Successfully selected network %s"%network[ESSID] )
			return True

		elif network[TYPE]==CABLE:
			try:
				self.__get_nm_dbus_object().setActiveDevice(
					dbus.ObjectPath( network[DEVICE_PATH] ) )
			except dbus.DBusException,e:
				debug.error( 3, "CLIENT",
					"%s: %s"%(NAME_DEVICE_TYPE[CABLE],e) )
				return False
			network[TRUSTED]=True
			self.main.network_activated( network=network )
			debug.info( 2, "CLIENT",
				"Successfully selected device %s"%network[INTERFACE] )
			return True

		else:
			debug.error( 8, "CLIENT", "Unknown type: %s",network[TYPE] )
			return False

