
#
FILES_PATH = '/usr/share/pyNetworkManager/'

# Status
UNKNOWN	= 'Unknown status'
WAITING	= 'Waiting for device to activate'
ACTIVE	= 'A network is active'
PASSIVE	= 'No active network'
NAME_STATUS = {
	UNKNOWN	: "Unknown status",
	WAITING	: 'Waiting for device to activate',
	ACTIVE	: 'A network is active',
	PASSIVE	: 'No active network',
	}

# Network types
UNKWNOWN	= 0
CABLE		= 1
WIRELESS	= 2
NAME_DEVICE_TYPE = {
	UNKWNOWN:	'unknown',
	WIRELESS:	'wireless',
	CABLE: 		'cable',
	}

# Activation steps
ACT_STAGE_UNKNOWN          = 0
ACT_STAGE_DEVICE_PREPARE   = 1
ACT_STAGE_DEVICE_CONFIG    = 2
ACT_STAGE_NEED_USER_KEY    = 3
ACT_STAGE_IP_CONFIG_START  = 4
ACT_STAGE_IP_CONFIG_GET    = 5
ACT_STAGE_IP_CONFIG_COMMIT = 5
ACT_STAGE_ACTIVATED        = 6
ACT_STAGE_FAILED           = 7
ACT_STAGE_CANCELLED        = 8


# Properties for devices and networks
INTERFACE       = 'DeviceInterface'
TYPE            = 'DeviceType'
DEVICE_PATH     = 'Path'
DEVICE_CAPS     = 'DeviceCapabilities'	# encryptions device can handle

ACTIVE          = 'Active'		# device/network is used
ENABLED         = 'Enabled'	# device has a signal

BSSIDS          = 'Bssids'


NETWORK_PATHS   = 'NetworkPaths'
NETWORK_PATH    = 'NetworkPath'
ACTIVE_NETWORK_PATH  = 'ActiveNetworkPath'	# device property

NETWORK_CAPS    = 'NetworkCapabilities'	# type of encryption

ESSID           = 'Essid'
STRENGTH        = 'Strength'
ENCRYPTED       = 'Encrypted'
TRUSTED         = 'Trusted'
LAST_USED       = 'LastTimeUsed'
SECURITY        = 'Security'

from securityconstants import NM_802_11_CAP_PROTO_NONE
import security

#####################################################################

def active_to_string( active ):
	if active: return "active "
	else: return      "passive"
def enabled_to_string( active ):
	if active: return "enabled "
	else: return      "disabled"

class Device( dict ):
	def __init__( device, dev_path ):
		dict.__init__( device, {
				DEVICE_PATH   : dev_path,
				INTERFACE     : False,
				TYPE          : UNKNOWN,
				NETWORK_PATHS : [],
				ACTIVE        : False,
				ENABLED       : False,
				TRUSTED       : True,
				STRENGTH      : 0,
				LAST_USED     : 0,
				DEVICE_CAPS   : 0,
			} )
	"""
	def __str__( device ):
		return " Interface:%5s %s %s %s "%(
			device[INTERFACE],
			active_to_string( device[ACTIVE] ),
			enabled_to_string( device[ENABLED] ),
	"""

	def complement( device, p ):
		""" device->getProperties
			 0 DBUS_TYPE_OBJECT_PATH, &op,
			 1 DBUS_TYPE_STRING, &iface,
			 2 DBUS_TYPE_UINT32, &type,
			 3 DBUS_TYPE_STRING, &udi,
			 4 DBUS_TYPE_BOOLEAN,&active,
			 5 DBUS_TYPE_UINT32, &act_stage,
			 6 DBUS_TYPE_STRING, &ip4_address,
			 7 DBUS_TYPE_STRING, &subnetmask,
			 8 DBUS_TYPE_STRING, &broadcast,
			 9 DBUS_TYPE_STRING, &hw_addr_buf_ptr,
			10 DBUS_TYPE_STRING, &route,
			11 DBUS_TYPE_STRING, &primary_dns,
			12 DBUS_TYPE_STRING, &secondary_dns,
			13 DBUS_TYPE_INT32,  &mode,
			14 DBUS_TYPE_INT32,  &strength,
			15 DBUS_TYPE_BOOLEAN,&link_active,
			16 DBUS_TYPE_INT32,  &speed,
			17 DBUS_TYPE_UINT32, &capabilities,
			18 DBUS_TYPE_UINT32, &type_capabilities,
			19 DBUS_TYPE_STRING, &active_network_path,
			20 DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &networks, num_networks,
		"""
		device.update( {
			DEVICE_PATH   : p[0],
			INTERFACE     : p[1],
			TYPE          : p[2],
			ACTIVE        : p[4],
			ENABLED       : p[15],
			NETWORK_PATHS : list(p[20]),
			STRENGTH      : p[14],
			DEVICE_CAPS   : p[18],
			ACTIVE_NETWORK_PATH : p[19],
		} )
		return device

#####################################################################

class Network( Device ):
	def __init__( network, dev_path, net_path, essid='' ):
		Device.__init__( network, dev_path )
		network.update( {
				DEVICE_PATH  : dev_path,
				NETWORK_PATH : net_path,

				TYPE         : WIRELESS, # for now all these are wireless

				BSSIDS       : [],

				ENCRYPTED    : False,
				SECURITY     : security.NONE(),
				NETWORK_CAPS : 0,

				LAST_USED    : 0,

				ESSID        : essid,
				STRENGTH     : 0,
				TRUSTED      : False,

				ACTIVE       : False,
			} )

	def add_bssid( network, bssids ):
		if type( bssids )==str:
			if bssids not in network[BSSIDS]:
				network[BSSIDS].append( bssids )
		elif type( bssids ) in ( list, tuple ):
			for bssid in bssids:
				network.add_bssid( bssid )

	def complement( network, dev_p, net_p ):
		""" network->getProperties
			0 DBUS_TYPE_OBJECT_PATH, &op,
			1 DBUS_TYPE_STRING,  &essid,
			2 DBUS_TYPE_STRING,  &hw_addr_buf_ptr,
			3 DBUS_TYPE_INT32,   &strength,
			4 DBUS_TYPE_DOUBLE,  &freq,
			5 DBUS_TYPE_INT32,   &rate,
			6 DBUS_TYPE_INT32,   &mode,
			7 DBUS_TYPE_INT32,   &capabilities,
			8 DBUS_TYPE_BOOLEAN, &broadcast,
		"""
		Device.complement( network, dev_p )
		network.update( {
			NETWORK_PATH  : net_p[0],
			ESSID         : net_p[1],
			STRENGTH      : net_p[3],
			NETWORK_CAPS  : net_p[7],
		} )
		network[ENCRYPTED] = not ( net_p[7] & NM_802_11_CAP_PROTO_NONE )
		network.add_bssid( net_p[2] )
		if network[NETWORK_PATH]==network[ACTIVE_NETWORK_PATH]:
			network[ACTIVE]    = True
		else:
			network[ACTIVE]    = False
		return network
