# -*- encoding: utf8

"""
Read and write conf-file

CONFIG_FILE:
[YY-MM-DD HH:MM:SS]
	DeviceName=eth1
	bssid=[0:1:2:3:4, 2:3:4:5]
	lastTimeUsed=unixtimestamp
	security = ,

FIX:
	security array isn't of right types!

"""

from configobj import ConfigObj
import time		# for last time used and id's
import os		# to get path to home directory

from constants import *
from security import deserialize
import debug

import xdg.DesktopEntry
CONFIG_DIR = xdg.DesktopEntry.xdg_config_dirs[0]+"/pyNetworkManager/"

# WIRELESS
########################################################

CONFIG_WIRELESS_FILE = 'stored-wireless-networks.conf'

class init:
	def __init__( self, pynm ):
		self.pynm = pynm

		if not os.path.isdir( CONFIG_DIR ):
			# Create configuration directory
			debug.info(5, "DISK",
				"Creating directory for configuration files %s"%CONFIG_DIR )
			os.makedirs( CONFIG_DIR )
		self.__configobj = ConfigObj( CONFIG_DIR+CONFIG_WIRELESS_FILE )

		self.pynm.known_networks.listen( self.list_known_networks )
		self.pynm.network_activated.listen( self.on_user_activate )
		self.pynm.complement_from_disk.listen( self.complement_network )

#  events
	def on_user_activate( self, network=False ):
		if network.get(TYPE)==WIRELESS:
			self.pynm.status_changed.listen( self.on_status_changed, network )
		else:
			self.pynm.status_changed.remove( self.on_status_changed )

	def on_status_changed( self, selected, status=False, network=False ):
		if status==ACTIVE:
			if network \
			and network.get(TYPE)==WIRELESS \
			and selected.get(NETWORK_PATH) == network.get(NETWORK_PATH):
				self.__add( network )
			else:
				self.pynm.status_changed.remove( self.on_status_changed )

# methods

	def list_known_networks( self ):
		net_list = []
		for net_id in self.__configobj.keys():
			net = self.__configobj.get(net_id)
			network = Network(None,None)
			network.update( {
				ESSID       : unicode( net[ESSID] ),
				BSSIDS      : net[BSSIDS],
				TRUSTED     : bool(net[TRUSTED]),
				LAST_USED   : int(net[LAST_USED]),
				SECURITY    : security.deserialize( net[SECURITY] ),
				TYPE        : int(net[TYPE]),
			} )
			net_list.append(network)
		return net_list

	def complement_network( self, network=False ):
		" add info about network if it exists "
		if not network \
		or network[TYPE]!=WIRELESS:
			return False
		id = self.__id( network )
		if id:
			debug.info(8,'CONF','Stored info about network is added')
			net = self.__configobj.get(id)
			sec = security.deserialize( net[SECURITY] )
			if sec: network[SECURITY] = sec
			network.update( {
				TYPE        : WIRELESS,
				LAST_USED   : net[LAST_USED],
				TRUSTED     : net[TRUSTED],
				} )
			network.add_bssid( net[BSSIDS] )
			return True
		return False

# helpers
	def __add( self, network ):
		" stores(replaces) a network to configuration and file"
		id = self.__id( network )
		if id: del self.__configobj[id]
		id = self.__generate_id()

		self.__configobj[id]={
				ESSID       : network[ESSID],
				BSSIDS      : network[BSSIDS],
				TRUSTED     : network[TRUSTED],
				LAST_USED   : int( round( time.time() ) ),
				SECURITY    : network[SECURITY].serialize(),
				TYPE        : network[TYPE],
			}
		self.__configobj.write()

	def __id( self, network ):
		for id in self.__configobj.keys():
			if network.get(ESSID)==self.__configobj[id][ESSID]:
				return id
		return False

	def __generate_id( self ):
		while 1:
			id = time.strftime("%Y-%m-%d %H:%M:%S",time.localtime())
			if id not in self.__configobj.keys(): return id

########################################################
