# -*- coding: utf-8

"""
nm: key( Network )?
 conf: known encryption( Network )?
 uigtks: security_from_network( Network )?
 uigtks: userdialog( Security )?
  user ok/abort
"""

#external libraries
import pygtk
pygtk.require("2.0")
import gtk
import gtk.glade

# internal libraries
from pyNM.constants import *
import pyNM.security
from pyNM.securityconstants import *		# constants and network
import pyNM.debug								# debugging

GLADE_SECURITY_DIALOG = FILES_PATH+'/pyNM/gtk2/setup_net.glade'
global running_dialog
running_dialog = False

################################################################

def init(main):
	main.ask_for_network_key.listen( get_key_for_network )
	main.abort_network_key_question.listen( get_key_abort )

################################################################

def gdialog( new_dialog=False, close_dialog=False ):
	global running_dialog
	if   new_dialog:
		running_dialog = new_dialog
	elif close_dialog:
		if running_dialog:
			running_dialog.response(-1)
	else:
		return running_dialog

def get_key_for_network( network=None ):
	if network:
		try:
			net_sec = pyNM.security.security_from_caps( network )
		except pyNM.security.ErrorDeviceCipher, E:
			debug.error( 3, "GTK Security", E )
		except pyNM.security.ErrorCipherUnknown, E:
			debug.error( 3, "GTK Security", E )
		except pyNM.security.ErrorCipherType, E:
			debug.error( 3, "GTK Security", E )
		else:
			gdialog( new_dialog=DialogView( network[ESSID], sec=net_sec ) )
			if gdialog().run():
				network[SECURITY] = gdialog().get_security()
			else:
				network[SECURITY] = pyNM.security.NONE()
			gdialog().destroy()
	return network

def get_key_for_new_network():	# needs interface info!
	gdialog( new_dialog=DialogView( ) )
	network = None
	if gdialog().run():
		network = Network(None,None)
		network[ESSID] = gdialog().get_name()
		network[INTERFACE] = gdialog().get_interface()
		network[SECURITY] = gdialog().get_security().serialize()
	gdialog().destroy()
	return network

def get_key_abort():
	pyNM.debug.info( 6, "GTK Security", "Aborts keyquestion" )
	gdialog( close_dialog=True )


#############################################################
class DialogView( gtk.glade.XML ):
	def __init__( self, essid=False, interface=False, sec=False ):
		gtk.glade.XML.__init__( self, GLADE_SECURITY_DIALOG )
		self.dialog = self.get_widget('setup_net')
		self.ok_button = self.get_widget('ok_button' )
		self.error_color = gtk.gdk.color_parse("red")
		self.notebook = self.get_widget('notebook')

		if essid:
			self.get_widget('networkName').set_text(essid)
			self.get_widget('networkName').set_sensitive( False )

		# Securities, in RIGHT order ( same as tabs )
		self.securities = [
			pyNM.security.NONE(),
			pyNM.security.WEP(),
			pyNM.security.WPA_PSK(),
			pyNM.security.WPA_EAP(),
		]
		self.security = self.securities[0]

		# for not entry widgets, like radiobuttons
		self.widget_name_to_value = {
			'encryptionWEPopen' : IW_ALG_OPEN_SYSTEM,
			'encryptionWEPshared' : IW_ALG_SHARED_KEY,

			'Password' : NM_AUTH_TYPE_WEP104,
			'40 bit' : NM_AUTH_TYPE_WEP40,
			'104 bit' : NM_AUTH_TYPE_WEP104,
		}

		# setup selected security type
		if sec: self.__setup_security( sec )

		# setup events
		self.signal_autoconnect( {
				'on_toggle_readable' : self.on_toggle_readable,

				'on_name_changed' : self.on_name_changed,

				'changed_key' : ( self.__set, 'key' ), # set_and_mark(
				'changed_type' : ( self.__set, 'type' ),
				'changed_auth_alg' : ( self.__set, 'auth_alg' ),
				'changed_version' : ( self.__set, 'version' ),
				'changed_encryption_type' : self.on_changed_encryption_type,
			} )

#________________________________________________________________________
	# public methods

	def run( self ):
		" Shows dialog and return user entered encryption from it "
		return 1==self.dialog.run()

	def get_security( self ):
		return self.security

	def get_name( self ):
		return self.get_widget('networkName').get_text()

	def get_interface( self ):
		return False

	def destroy( self ):
		self.dialog.destroy()

#________________________________________________________________________
	# on events

	def on_name_changed( self, *rest ):
		self.check()

	def on_changed_encryption_type( self, notebook, something, page ):
		" Select wich type of encryption that is selected (and active) "
		self.security = self.securities[ page ]
		self.check()

	def on_toggle_readable( self, widget ):
		" Show widget text as text or symbol ( for passwords )"
		widget.set_visibility( not widget.get_visibility() )

	def check( self ):
		if	len( self.get_widget('networkName').get_text() )>0 \
		and len(self.security.check())==0:
			self.ok_button.set_sensitive( True )
		else:
			self.ok_button.set_sensitive( False )

	def __set( self, widget, name, *rest ):
		self.security.set( name, self.get_widget_value( widget ) )
		self.check()

	"""
	def set_and_mark( self, widget, name, *rest ):
		if self.security.set( name, self.get_widget_value( widget ) ):
			widget.parent.modify_bg(gtk.STATE_NORMAL, None)
		else:
			widget.parent.modify_bg(gtk.STATE_NORMAL,
				self.error_color )
		self.check()
	"""

	def get_widget_value( self, widget ):
		if   type( widget )==gtk.Entry:
			return widget.get_text()
		elif type( widget )==gtk.RadioButton:
			return self.widget_name_to_value.get( widget.name )
		elif type( widget )==gtk.ComboBox:
			return self.widget_name_to_value.get( widget.get_active_text() )

#________________________________________________________________________
	# on setup

	def __select_radio( self, widgets, selected ):
		for index in range(0,len(widgets)):
			if index==selected:
				widgets[index].set_active( True )
			widgets[index].set_sensitive( False )

	def __select_list( self, widget, selected, keep_sensitive=False ):
		widget.set_active( selected )
		if not keep_sensitive:
			widget.set_sensitive( False )

	def __select_security( self, page ):
		self.security = self.securities[ page ]
		self.notebook.set_current_page( page )
		self.notebook.set_show_tabs( False )

	def __setup_security( self, sec ):
		if   type(sec)==pyNM.security.WEP:
			self.__select_security( 1 )
			self.setup_WEP( sec )
		elif type(sec)==pyNM.security.WPA_PSK:
			self.__select_security( 2 )
			self.setup_WPA_PSK( sec )
		else:
			debug.error( 5, "GTK Security",
				"Encryption type %s not supported yet"%
				NM_AUTH_TYPE_NAMES.get( sec.get('type') ) )
		self.check()

	# WEP:
	def setup_WEP( self, sec ):
		self.get_widget('encryptionWEPkey').set_text(sec['key'])

		# Could this be handled automatic? else, what types is there
		self.__select_list(
			self.get_widget('encryptionWEPkeytype'),
			0, keep_sensitive=True )

		auth_algs = [ IW_ALG_OPEN_SYSTEM, IW_ALG_SHARED_KEY ]
		self.__select_radio(
			self.get_widget('encryptionWEPopen').get_group(),
			auth_algs.index( sec['auth_alg'] ) )

	# WPA_PSK:
	def setup_WPA_PSK( self, sec ):
		self.get_widget('encryptionWPA_PSKkey').set_text(sec['key'])
		versions = [ IW_AUTH_WPA_VERSION_WPA, IW_AUTH_WPA_VERSION_WPA2 ]
		self.__select_radio(
			self.get_widget('encryptionWPA_PSKv1').get_group(),
			versions.index( sec['version'] ) )
		protocols = [
			NM_AUTH_TYPE_WPA_PSK_AUTO,
			NM_AUTH_TYPE_WPA_PSK_TKIP,
			NM_AUTH_TYPE_WPA_PSK_CCMP ]
		self.__select_list(
			self.get_widget('encryptionWPA_PSKprotocol'),
			protocols.index( sec['type'] ) )

		#self.key_management = key_management

################################################################

	#print get_key_for_network( {DEVICE_CAPS:61647, NETWORK_CAPS:12290} )
	#print get_key_for_network( {DEVICE_CAPS:61647, NETWORK_CAPS:28740} )
