# -*- encoding: utf8

"""
ToDo:
1: Add essid to tooltip
2: Fix some statusvariables and
	store status in NM (again?)
	And fix some conversion from
	those to text.
	"Not connected","Connecting","Connected","Error"
3: Set unconnected as startup status
4: Add tooltip to menuitems
"""

#external libraries
import pygtk
pygtk.require("2.0")
import gtk

# if __name__=="__main__": fix paths

# internal libraries
from pyNM.constants import *   # constants and network
import pyNM.debug              # debugging
import pyNM.security

import securityui

##########################################################
# ICON showing current status

# Default = ICON_PATH = FILES_PATH+'icons/';
ICON_PATH = FILES_PATH+'icons/';

ICON_ERROR = ICON_PATH+'error.png'
ICON_WAIT = ICON_PATH+'working.png'
ICON_UNKNOWN = ICON_PATH+'unknown.png'
ICON_WIRELESS = ICON_PATH+'wireless.png'
ICON_CABLE = ICON_PATH+'cable.png'
ICON_QUALITY_1 = ICON_PATH+'quality_1.png'
ICON_QUALITY_2 = ICON_PATH+'quality_2.png'
ICON_QUALITY_3 = ICON_PATH+'quality_3.png'
ICON_QUALITY_4 = ICON_PATH+'quality_4.png'

def qualityIcon(quality):
	if quality>75:
		return ICON_QUALITY_4
	if quality>50:
		return ICON_QUALITY_3
	if quality>25:
		return ICON_QUALITY_2
	return ICON_QUALITY_1

###############################################3

class Icon(gtk.Image):
	def __init__(self, main):
		gtk.Image.__init__(self)
		self.set_from_file(ICON_WAIT)
		self.show()

		self.main = main
		self.main.status_changed.listen( self.on_status_changed )

	def set_from_file( self, icon ):
		gtk.Image.set_from_file( self, icon )

	def on_status_changed(self,status=UNKNOWN,network=False ):
		" Check deviceName and strength if any active device "
		self.main.strength_changed.remove( self.on_strength_changed )
		if status==ACTIVE:
			if network[TYPE]==CABLE:
				self.set_from_file(ICON_CABLE)
			elif network[TYPE]==WIRELESS:
				self.set_from_file(qualityIcon(network[STRENGTH]))
				self.main.strength_changed.listen( self.on_strength_changed )
			else:
				pyNM.debug.error(3,'GTK Icon','Unknown device type' )
		elif status==WAITING:  self.set_from_file(ICON_WAIT)
		elif status==PASSIVE:  self.set_from_file(ICON_ERROR)
		else:                  self.set_from_file(ICON_UNKNOWN)

	def on_strength_changed(self, strength=False):
		if strength:	# !=False and > 0
			self.set_from_file(qualityIcon(strength))

##########################################################

class Speedbars:
	def __init__(self, main):
		self.main = main

		self.barRx = gtk.ProgressBar()
		self.barRx.set_orientation(2)
		self.barRx.set_size_request(7,-1)
		#self.barRx.set_colormap(colors)
		self.barRx.show()

		self.barTx = gtk.ProgressBar()
		self.barTx.set_orientation(2)
		self.barTx.set_size_request(7,-1)
		self.barTx.show()

		self.set_inactive()

		main.status_changed.listen( self.on_status_changed )

	def on_status_changed( self, status=False, network=False ):
		if status==ACTIVE:
			self.main.speed_changed.listen( self.on_speed_changed )
		else:
			self.main.speed_changed.remove( self.on_speed_changed )
			self.set_inactive()

	def on_speed_changed(self,rx,tx):
		if rx and tx:
			self.barRx.set_fraction(rx['percent'])
			self.barTx.set_fraction(tx['percent'])
		else:
			self.set_inactive()

	def set_inactive( self ):
		self.barRx.set_fraction(0.5)
		self.barTx.set_fraction(0.5)

##########################################################

class Tooltip:
	def __init__( self, main, target_widget ):
		self.__target = target_widget

		self.__Tooltips = gtk.Tooltips()

		self.__tip_title = 'Starting...'
		self.__tip_speed = ''
		self.__setTip()

		main.status_changed.listen( self.on_status_changed )
		main.speed_changed.listen( self.on_speed_changed )
		main.nm_error.listen( self.on_error )

	def on_speed_changed( self, rx, tx ):
		if rx and tx:
			self.__tip_speed = u'rx: %2dkB/s tx: %2dkB/s'%(
					rx['per sec']/1024,
					tx['per sec']/1024 )
		else:
			self.__tip_speed = False
		self.__setTip()

	def on_status_changed(self, status=UNKNOWN, network=None ):
		if status==ACTIVE:
			if network[TYPE]==WIRELESS:
				self.__tip_title = u'%s (%i%%)'%(
					network[ESSID], network[STRENGTH] )
			else:
				self.__tip_title = u'%s'%(
					NAME_DEVICE_TYPE.get(network[TYPE]) )
		elif status==WAITING:   self.__tip_title = u'Waiting'
		elif status==PASSIVE:   self.__tip_title = u'Disconnected'
		else:                   self.__tip_title = u'Unknown'
		self.__setTip()

	def on_error(self, error):
		self.__tip_title = u'Error: %s'%error
		self.__setTip()

	def __setTip( self ):
		tip = self.__tip_title
		if self.__tip_speed:
			tip += u'\n'+self.__tip_speed
		self.__Tooltips.set_tip( self.__target, tip )

##########################################################

class Menu(gtk.Menu):
	def __init__( self, main ):
		gtk.Menu.__init__(self)
		self.main = main

		self.__tooltips = gtk.Tooltips()

	def show(self,sender):
		self.__populate()
		self.popup(None,None,None,0,0)	#event.button,event.time)

	def __Clear(self,item):
		self.remove(item)

	def __populate(self):
	# Clear menu
		self.foreach(self.__Clear)

	# List all connections
		devices = self.main.available_devices()
		if devices:
			for device in devices:
				networks = self.main.available_networks( device=device )
				if networks: # len(networks)?
					" Wireless device with networks "
					for network in networks:
						self.main.complement_from_nm( network=network )
						self.main.complement_from_disk( network=network )
						self.add_network_connection( device, network )
				elif device[TYPE]==CABLE and device[ENABLED]:
						" Cable device with cable "
						self.add_cable_connection( device )
				else:
						" device without cable or networks "
						self.add_disabled_connection( device )

		# Add standard elements:

		separator = gtk.MenuItem()
		self.append(separator)
		separator.show()

		#item = gtk.MenuItem('Stored connections')
		#menu.append( item )
		#item.show()
		#item.connect("activate", self.networkslist.show )

		item = gtk.MenuItem('Custom Connection...')
		self.append(item)
		item.show()
		item.connect("activate", self.custom_connection)

	def add_disabled_connection( self, device ):
		box = gtk.HBox()
		box.show()
		self.item_active( box, False )
		self.item_name( box, NAME_DEVICE_TYPE.get( device[TYPE] ) )
		self.item_device_type( box, device[TYPE] )
		item = gtk.MenuItem( None )
		item.add( box )
		self.append( item )
		item.set_sensitive( False )
		item.show()

	def add_cable_connection( self, device ):
		box = gtk.HBox()
		box.show()
		self.item_active( box, device[ACTIVE] )
		self.item_name( box, NAME_DEVICE_TYPE[ CABLE ] )
		#self.item_encrypted( box, False )
		#self.item_strength( box, 0 )
		self.item_device_type( box, CABLE )
		item = gtk.MenuItem( None )
		item.add( box )
		item.connect('activate', self.onConnect,device )
		self.append( item )
		item.set_sensitive( True )
		item.show()
		self.__tooltips.set_tip( item,
			"%s"%(device[INTERFACE]) )

	def add_network_connection( self, device, network ):
		box = gtk.HBox()
		box.show()
		self.item_active( box, network[ACTIVE] )
		self.item_name( box, network[ESSID] )
		self.item_encrypted( box, network[ENCRYPTED] )
		self.item_strength( box, network[STRENGTH] )
		self.item_device_type( box, WIRELESS )
		item = gtk.MenuItem(None)
		item.add( box )
		item.connect('activate', self.onConnect,network )
		self.append( item )
		item.set_sensitive( True )
		item.show()
		self.__tooltips.set_tip( item,
			"%s %s %i%%"%(
				network[INTERFACE],
				unicode(network[ENCRYPTED]),
				network[STRENGTH]
		) 	)
		#import pyNM.security

	#______ ICONS ___________________________

	def Icon(self,stock=None,file=None):
		icon=gtk.Image()
		icon.set_size_request(16,16)
		if stock:	icon.set_from_stock(stock,gtk.ICON_SIZE_MENU)
		elif file:	icon.set_from_file(file)
		icon.show()
		return icon

	def item_encrypted(self, box, encrypted):
		if encrypted: icon=self.Icon(stock=gtk.STOCK_DIALOG_AUTHENTICATION)
		else: icon=self.Icon()
		box.pack_start( icon, 0 )

	def item_active(self, box, active):
		if active: icon = self.Icon(stock=gtk.STOCK_YES)
		else: icon = self.Icon()
		box.pack_start( icon, 0 )

	def item_strength(self, box, strength):
		if strength>0: icon = self.Icon(file=qualityIcon(strength))
		else: icon = self.Icon()
		box.pack_start( icon, 0 )

	def item_device_type(self, box, deviceType):
		if deviceType==WIRELESS: icon = self.Icon(file=ICON_WIRELESS)
		elif deviceType==CABLE: icon = self.Icon(file=ICON_CABLE)
		else: icon = self.Icon(file=ICON_UNKNOWN)
		box.pack_start( icon, 0 )

	#_________ LABELS _________________________

	def item_name(self,box,name):
		if name: label=gtk.Label(name)
		else:    label=gtk.Label()
		label.set_alignment(0,0.5)
		label.show()
		box.pack_start( label, 1,1,6 )

	#____________ EVENTS ________________________________________

	def onConnect(self,event,network=None ):
		self.main.activate_network( network=network )

	#_____________________________________________________

	def custom_connection( self, *rest ):
		network = securityui.get_key_for_new_network()
		if network:
			pyNM.debug.info( 3, "GTK Custom",
				"User created network:\n%s\n"%unicode(network) )
			self.main.activate_network( network=network )


#######################################################
