# -*- coding: utf-8

"""
# nm-ap-security-wep.c "

nm: key( Network )?
 conf: known encryption( Network )?
 uigtks: security_from_network( Network )?
 uigtks: userdialog( Security )?
  user ok/abort

devcaps,netcaps => device and network information
"""

from constants import *
from securityconstants import *
import debug

###############################################################

class ErrorDeserialize( Exception ):		pass

class ErrorDeviceCipher( Exception ):		pass
class ErrorCipherUnknown( Exception ):		pass
class ErrorCipherType( Exception ):			pass


# Find out type of encryption a network needs
###############################################################

def security_from_caps( network ):

	# capabilities ( that current network needs )
	caps = network.get( NETWORK_CAPS )

	debug.info(3,"SECURITY", "Desciding cipher type... %i:%i"%
		(network[DEVICE_CAPS],caps) )

	# compare with what the device can handle
	caps &= network.get(DEVICE_CAPS)

	# Check if device handles this encryption
	if not ( caps ):
		raise ErrorDeviceCipher( "Device doesn't support cipher" )

	sec = NONE()

	# None
	if ( caps & NM_802_11_CAP_PROTO_NONE ):
		debug.info(8, "SECURITY","Cipher type = None" )

	# WEP
	elif ( caps & NM_802_11_CAP_PROTO_WEP ):
			sec = WEP()
			if caps & NM_802_11_CAP_CIPHER_WEP40:
				sec.set('type', NM_AUTH_TYPE_WEP40 )
			elif caps & NM_802_11_CAP_CIPHER_WEP104:
				sec.set('type', NM_AUTH_TYPE_WEP104 )
			else:
				debug.error(6,"SECURITY","Unknown WEP type")

			if caps & IW_ALG_OPEN_SYSTEM:
				sec.set('auth_alg', IW_ALG_OPEN_SYSTEM )
			else:
				sec.set('auth_alg', IW_ALG_SHARED_KEY )

	# WPA
	elif ( caps & NM_802_11_CAP_PROTO_WPA ) \
	  or ( caps & NM_802_11_CAP_PROTO_WPA2 ):

		# WPA_EAP
		if caps & NM_802_11_CAP_KEY_MGMT_802_1X:
			debug.info(8, "SECURITY","Cipher type = WPA EAP" )
			#sec = WPA_EAP()

		# WPA_PSK
		elif caps & NM_802_11_CAP_KEY_MGMT_PSK:
			sec = WPA_PSK()

			if caps & NM_802_11_CAP_CIPHER_CCMP:
				sec.set('type', NM_AUTH_TYPE_WPA_PSK_CCMP )
			elif caps & NM_802_11_CAP_CIPHER_TKIP:
				sec.set('type', NM_AUTH_TYPE_WPA_PSK_TKIP )
			else:
				sec.set('type', NM_AUTH_TYPE_WPA_PSK_AUTO )

			if caps & NM_802_11_CAP_PROTO_WPA:
				sec.set('version', IW_AUTH_WPA_VERSION_WPA2 )
			elif caps & NM_802_11_CAP_PROTO_WPA:
				sec.set('version', IW_AUTH_WPA_VERSION_WPA )
			else:
				debug.error(6,"SECURITY","Unknown WPA version")

		else:
			raise ErrorCipherType( "Unknown WPA type" )

	# Unknown
	else:
		raise ErrorCipherUnknown( "Unknown cipher" )

	return sec

# Template, None
##############################################################3

DBUS_SIGNATURES = {
	'i':int,
	's':str,
}
class NONE( dict ):
	"""
		NONE encryption
		base class for all encryptions
	"""
	def __init__( self ):
		self['type'] = NM_AUTH_TYPE_NONE
		self.serie = [ 'type' ]
		self.dbusSignature = 'i'

	def deserialize( self, serie, fix_types=False ):
		for i in range(len(self.serie)):
			if not len(serie): break
			value = serie.pop(0)
			if fix_types:
				value = DBUS_SIGNATURES[ self.dbusSignature[i] ] ( value )
			self[self.serie[i]]=value

	def serialize( self ):
		list = []
		for item in self.serie:
			list.append( self[item] )
		return list

	def set( self, name, value ):
		if name in self.keys():
			self[name]=value
			return True
		else:
			return False

	def check( self ):
		errors = []
		if self['type']!=NM_AUTH_TYPE_NONE:
			errors.append('type')
		return errors

	#def check_string( self, string, min_length, max_length, allowed_chars )

	def __str__( self ):
		return "%s: %s"%(
			NM_AUTH_TYPE_NAMES.get(self['type']), unicode(self.items()) )

# WEP
##############################################################3
class WEP( NONE ):
	"""
		Passphrase ( WEP40/WEP104 )
					ASCII keylength=1-64
		WEP40		ASCII keylength=5
					HEX   keylength=10
		WEP104	ASCII keylength=13
					HEX   keylength=26
		AUTO, detect WEP40/WEP104 depending on key length ?!
	"""
	def __init__( self ):
		dict.__init__( self, {
			'type'      : NM_AUTH_TYPE_WEP104,
			'key'       : '',
			'auth_alg'  : IW_ALG_OPEN_SYSTEM,
		} )
		self.serie = ( 'type', 'key', 'auth_alg' )
		self.dbusSignature = 'isi'

	def check( self ):
		errors = []

		couldbehex = True
		for char in self['key']:
			if char not in HEX_CHARS:
				couldbehex = False
				break
		if self['type']==NM_AUTH_TYPE_WEP40:
			if len(self['key'])!=5 \
			and ( len(self['key'])!=10 or not couldbehex ):
				errors.append('key')
		if self['type']==NM_AUTH_TYPE_WEP104:
			if len(self['key'])!=13 \
			and ( len(self['key'])!=26 or not couldbehex ):
				errors.append('key')

		if not self['type'] in (NM_AUTH_TYPE_WEP40,NM_AUTH_TYPE_WEP104):
			errors.append('type')
		if not self['auth_alg'] in (IW_ALG_OPEN_SYSTEM,IW_ALG_SHARED_KEY):
			errors.append('auth_alg')
		return errors

# WPA PSK
###################################################################

class WPA_PSK( NONE ):
	def __init__( self ):
		dict.__init__( self, {
			'type'      : NM_AUTH_TYPE_WPA_PSK_AUTO,
			'key'       : '',
			'version'   : IW_AUTH_WPA_VERSION_WPA,
			'key_mgmt'  : IW_AUTH_KEY_MGMT_802_1X,
		} )
		self.serie = ( 'type', 'key', 'version', 'key_mgmt' )
		self.dbusSignature = 'isii'

	def check( self ):
		errors = []
		if not self['type'] in (
				NM_AUTH_TYPE_WPA_PSK_AUTO,
				NM_AUTH_TYPE_WPA_PSK_TKIP,
				NM_AUTH_TYPE_WPA_PSK_CCMP ):
			errors.append('type')
		if not self['version'] in (
				IW_AUTH_WPA_VERSION_WPA,
				IW_AUTH_WPA_VERSION_WPA2 ):
			errors.append('version')
		if not self['key_mgmt'] in (
				IW_AUTH_KEY_MGMT_802_1X,
				IW_AUTH_KEY_MGMT_PSK ):
			errors.append('key_mgmt')
		if not len( self['key'] )>3:
			errors.append('key')
		return errors

# WPA EAP
##############################################################3
class WPA_EAP( NONE ):
	pass




# Find out a network from serialized data
##############################################################3

ENCRYPTIONS = {
	NM_AUTH_TYPE_NONE: NONE,
	NM_AUTH_TYPE_WEP40: WEP,
	NM_AUTH_TYPE_WEP104: WEP,
	NM_AUTH_TYPE_WPA_PSK_AUTO: WPA_PSK,
	NM_AUTH_TYPE_WPA_PSK_TKIP: WPA_PSK,
	NM_AUTH_TYPE_WPA_PSK_CCMP: WPA_PSK,
}

def deserialize( serie, fix_types=True ):
	if len(serie)==0: return False
	if fix_types: serie[0] = int(serie[0])	# Fix type definition
	sec = ENCRYPTIONS.get(serie[0])
	if sec:
		sec=sec()
		sec.deserialize( serie, fix_types=fix_types )
		if len(sec.check())==0:
			return sec
	debug.error(6,'SECURITY','Error serializing %s'%serie)
	return NONE()








# Test run
##############################################################3

if __name__=="__main__":
	debug.set_error_level(10)
	debug.set_info_level(10)
	sec = security_from_network( {DEVICE_CAPS:61647, NETWORK_CAPS:12290} )
	sec.set('key','0a2bA5f78f')
	errors = sec.check()
	if len(errors):
		debug.error(8, "SECURITY","Cipher errors: %s"%unicode(errors) )
	print sec
	print sec.serialize()
	print "----"
	sec = security_from_network( {DEVICE_CAPS:61647, NETWORK_CAPS:28740} )
	errors = sec.check()
	if len(errors):
		debug.error(8, "SECURITY","Cipher errors: %s"%unicode(errors) )
	print sec
	print sec.serialize()

