# -*- coding: utf-8

"""
pyNM.service

ToDo:
	* On creation, check if there already is one.
		If there is, warn and skip connection.
	* Maby put connection in a separate place to
		be able to handle failure in NM.

"""

# External libraries
import dbus
import dbus.service
import dbus.glib	# IMPORTANT

DBUS_SIGNATURES = {
	'i':dbus.Int32,
	's':dbus.String,
	'b':dbus.Boolean,
	}

# Own libraries
import security

from constants import *
import debug


# Server (info)
NMI_DBUS_SERVICE	= 'org.freedesktop.NetworkManagerInfo'
NMI_DBUS_PATH		= '/org/freedesktop/NetworkManagerInfo'
NMI_DBUS_IFACE 		= 'org.freedesktop.NetworkManagerInfo'

#######################################################################

class Service( dbus.service.Object ):
	""" Server	Svara på frågor från nm via dbus, NMI
		// gnome applet xxx.c
		* getNetworks
		* getNetworkProperties
		* updateNetworkInfo
		* getKeyForNetwork
		* cancelGetKeyForNetwork
		* getVPNConnections
		* getVPNConnectionProperties
		* getVPNConnectionVPNData
		* getVPNConnectionRoutes

		// nm-dbus-nmi.c
		* getKeyForNetwork
		* cancelGetKeyForNetwork
		* getNetworks
		* getNetworkProperties
		* updateNetworkInfo
	"""
	def __init__( self, main ):
		self.main = main

		name = dbus.service.BusName(NMI_DBUS_SERVICE,bus=dbus.SystemBus())
		dbus.service.Object.__init__( self, name, NMI_DBUS_PATH )

#######################################################################

	@dbus.service.method( NMI_DBUS_IFACE )
	def getNetworks( self,*rest ):
		"NM wants a list of all known networks => [ string:ESSID ]"
		debug.info( 5, "SERVER", "NM asks for known networks list" )

		message = dbus.Array( signature=dbus.Signature('s') )

		networks = self.main.known_networks()
		if networks:
			for network in networks:
				if network[TYPE]==WIRELESS \
				and network[TRUSTED]:
					message.append( network[ESSID] )
		debug.info( 8, "SERVER", unicode( message ) )
		return message

#######################################################################

	@dbus.service.method( NMI_DBUS_IFACE )
	def getNetworkProperties(self,essid,mabyDeviceType):
		"""
		NM wants information about a network

		=> str:ESSID     int:LAST_USED  bool:TRUSTED?
		   [ str:BSSID ] int:SECURITY.. int:DEVICE_TYPE?
		( security is one or more parameters describing security, 1==None )
		"""
		debug.info( 6, "SERVER",
			"NM asks information about network %s, %d"%(
			essid,mabyDeviceType) )

		nw=Network( '','',essid=essid)
		self.main.complement_from_disk( network=nw )

		net_sig = nw[SECURITY].dbusSignature
		serie = nw[SECURITY].serialize()

		msg = []
		msg.append( dbus.String( nw[ESSID] ) )
		msg.append( dbus.Int32( nw[LAST_USED] ) )
		msg.append( dbus.Boolean( nw[TRUSTED] ) )
		msg.append( dbus.Array( nw[BSSIDS], 's' ) )
		for i in range( len( serie ) ):
			msg.append( DBUS_SIGNATURES[ net_sig[i] ]( serie[i] ) )
		msg.append( dbus.Int32( nw[TYPE] ) )

		debug.info( 8, "SERVER", msg )
		return tuple(msg)

#######################################################################

	@dbus.service.method( NMI_DBUS_IFACE )
	def updateNetworkInfo(self,essid,automatic,bssid,*security):
		""" NM sends information about a network
		str:ESSID bool:automatic/trusted???(0) str:BSSID security..
		"""

		debug.info(7, "SERVER", "NM inform about network: %s, %s, %s, %s"%
			(essid,trusted,bssid,unicode(security)) )

		network = Network( False, False, essid=essid )
		network.add_bssid( bssid )
		network[ SECURITY ] = security

		self.main.network_info_update.trigger( network )

		debug.info( 9, "SERVER", "Network is: %s"%unicode(network) )

	@dbus.service.method( NMI_DBUS_IFACE )
	def getKeyForNetwork(self,dev_path,net_path,essid,attempts,newKey):
		"NM asks for a key to a network"
		debug.info(5, "SERVER",
			"NM asks for key to %s, attempt %i, new key = %s"%
				(essid,attempts,newKey) )

		network = Network( dev_path, net_path, essid=essid )
		self.main.complement_from_disk.request( network )
		network = self.main.ask_for_network_key.request( network=network )

		# check type, if none user aborted.
		if not network \
		or network[SECURITY]['type'] == security.NM_AUTH_TYPE_NONE:
			debug.info(4, "SERVER", "User aborted key request" )
			return False

		signature =  network[SECURITY].dbusSignature
		serialized = network[SECURITY].serialize()

		msg = []
		for i in range( len( serialized ) ):
			msg.append( DBUS_SIGNATURES[ signature[i] ]( serialized[i] ) )

		debug.info(4, "SERVER", "User answer on key request:" )
		debug.info(7, "SERVER", msg)
		return tuple(msg)

		"""
		# Skall returnera?
		# cipher type (wpa_psk_auto), cipher(key?)
		"""

#######################################################################

	@dbus.service.method( NMI_DBUS_IFACE )
	def cancelGetKeyForNetwork(self,*info):
		""" NM aborts key question (above) """
		debug.info(6, "SERVER", "NM aborts question for key %s",info )
		self.main.abort_network_key_question.trigger()

###############################################################

