# -*- encoding: utf8

# Automatisk uppdatering internt,
# skicka ut signal vid uppdatering

# Filtrera över tidigare värden

import time

SYS_PATH = "/sys/class/net/%s/statistics/"
BYTES_FILE = ( "rx_bytes", "tx_bytes" )

RX=0
TX=1

TOP_DECREASE=0.95

class Speed:
	def __init__(self, interfaceName, rxMax=-1, txMax=-1 ):
		self.__file = ( SYS_PATH % interfaceName + BYTES_FILE[RX],
						SYS_PATH % interfaceName + BYTES_FILE[TX] )
		self.__max = [rxMax,txMax]
		self.__lastTime = time.time()
		self.__last = [self.__read(RX),self.__read(TX)]
		self.__peak = [0,0]

	def setMax(self, rxMax=0, txMax=0):
		"If returned data is relative a set value or (-1) an automatic value"
		if rxMax != 0:
			self.__max[RX] = rxMax
		if txMax != 0:
			self.__max[TX] = txMax

	def __calculate(self,wich,perSec):
		# If no value
		if perSec==0: return 0
		# Return relative automatic max
		if self.__max[wich] <0:
			# decrease top value
			self.__peak[wich] = self.__peak[wich]*TOP_DECREASE
			# if new is larger, store
			if perSec > self.__peak[wich]: self.__peak[wich] = perSec
			# calculate relative value (always <=1)
			return perSec / self.__peak[wich]
		# Return relative manual max:
		else:
			rel = perSec / self.__max[wich]
			if rel > 1: return 1
			else: return rel

	def __read(self,wich):
		"Read total bytes transfered"
		try:
			f=open( self.__file[wich] )
			value = int( f.read() )
			f.close()
		except IOError:
			value = 0
		return value

	def getSpeed(self):
		" Returns rx and tx activity since last time checked"
		# Read current data
		now = time.time()
		rx = self.__read(RX)
		tx = self.__read(TX)

		# Calculate amount per second
		deltaTime = now - self.__lastTime
		rxPerSec = ( rx-self.__last[RX] ) / deltaTime
		txPerSec = ( tx-self.__last[TX] ) / deltaTime
		rxmod = self.__calculate(RX,rxPerSec)
		txmod = self.__calculate(TX,txPerSec)

		# Store for next time
		self.__lastTime = now
		self.__last = [ rx, tx ]

		return ( {	# RX
						'total':rx,
						'percent':rxmod,
						'per sec':rxPerSec,
						},
					{	# TX
						'total':tx,
						'percent':txmod,
						'per sec':txPerSec,
						}
					)


######################################################

if __name__=='__main__':
	import sys
	if len(sys.argv)==2:
		N = Speed(sys.argv[1])
		print    "    per second      |      total       "
		print    "    down      up    |     down     up  "
		while 1:
			(rx,tx) = N.getSpeed()
			print "%5dkB/s %5dkB/s |%.3f %3f"%(
				rx['per sec']/1024,tx['per sec']/1024,
				rx['total']/1024,tx['total']/1024 )
			time.sleep(1)
	else:
		print "sysNetworkMonitor.py interface"
